%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MP3 Bit Rate Quality Detection through Frequency Spectrum Analysis
%
%    This code performs SVM classification of frequency spectrum data in order to predict the bit rate of an mp3 file
%    Copyright (C) 2009  Brian D'Alessandro, Ph.D.
%    http://www.briandalessandro.com/
% 
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU Affero General Public License as
%    published by the Free Software Foundation, either version 3 of the
%    License, or (at your option) any later version.
% 
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU Affero General Public License for more details.
% 
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    For further information, please refer to the paper:
%
%    Brian D'Alessandro, Yun Q. Shi, "MP3 Bit Rate Quality Detection through Frequency Spectrum Analysis," 
%    in Proceedings of the 11th ACM Workshop on Multimedia and Security (MM&Sec 09), Princeton, NJ, September 7-8, 2009, pp. 57-62. 
%    DOI: 10.1145/1597817.1597828
% 
%    PDF available at: http://www.briandalessandro.com/about/cv.php
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ClassifyMp3BitRate
    addpath('osu-svm'); %path to SVM library

    %First, run bit rate differentiation SVM with training & test set
    [AlphaY,SVs,Bias,Parameters,nSV,nLabel] = Mp3BitRateDifferentiation();
    
    %Next, use the SVM generated previously & use it to test Transcoding Detection
    Mp3TranscodingDetection(AlphaY,SVs,Bias,Parameters,nSV,nLabel);


function [AlphaY,SVs,Bias,Parameters,nSV,nLabel] = Mp3BitRateDifferentiation()

    %Initialize parameters
    N_features = 100; %number of features to consider
    N_items = 2512;   %total number of songs
    N_testset = 2012; %number of songs used for testing
    N_trainset = 500; %number of songs used for training
    N_iterations = 5; %number of iterations for SVM, each with different divisions of test & train set

    %Load feature data
    load('mat\data_128');
    load('mat\data_192');
    load('mat\data_256');
    load('mat\data_vbr0');
    load('mat\data_320');

    %Keep only the first N_features, and then normalize each instance to the range -1 to 1.
    n_f128 = Scale(f128(1:N_features,:), -1, 1);
    n_f192 = Scale(f192(1:N_features,:), -1, 1);
    n_f256 = Scale(f256(1:N_features,:), -1, 1);
    n_fvb0 = Scale(fvbr0(1:N_features,:), -1, 1);
    n_f320 = Scale(f320(1:N_features,:), -1, 1);

    %Initialize average confusion matrix across the number of SVM iterations
    average_ConfMatrix_test=zeros(5,5);

    %Loop to run SVM N_iterations times
    for iteration=1:N_iterations

        %Create random permutation of the Number of songs, and divide into training set and test set indexes.
        tt = randperm(N_items);
        trainSet = tt(N_testset+1:N_items);    
        testSet = tt(1:N_testset);
        clear tt

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Construct training samples, as well as training labels
        %Label #1 = 128 kbps, Label #2 = 192 kbps, Label #3 = 256 kbps, Label #4 = VBR-0, Label #5 = 320 kbps
        training_sample = cat(2, n_f128(:, trainSet),  n_f192(:, trainSet),  n_f256(:, trainSet),  n_fvb0(:, trainSet),  n_f320(:, trainSet));
        training_label  = cat(2, 1*ones(1,N_trainset), 2*ones(1,N_trainset), 3*ones(1,N_trainset), 4*ones(1,N_trainset), 5*ones(1,N_trainset));    

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Construct test samples, as well as test labels
        test_sample = cat(2, n_f128(:, testSet),  n_f192(:, testSet),  n_f256(:, testSet),  n_fvb0(:, testSet),  n_f320(:, testSet));
        test_label  = cat(2, 1*ones(1,N_testset), 2*ones(1,N_testset), 3*ones(1,N_testset), 4*ones(1,N_testset), 5*ones(1,N_testset));                          

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Train SVM on training sample
        [AlphaY,SVs,Bias,Parameters,nSV,nLabel] = PolySVC(training_sample,training_label,2,1,1,1); %numeric parameters are: degree, C, gamma, coeff

        %Use the results of training to classify the Test sample & obtain performance
        [ClassRate_test,DecisionValue_test,Ns_test,ConfMatrix_test,PreLabels_test] = SVMTest(test_sample,test_label,AlphaY,SVs,Bias,Parameters,nSV,nLabel);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Record confusion matrix results of this iteration
        average_ConfMatrix_test = average_ConfMatrix_test + ConfMatrix_test / N_iterations;
    end

    %Display Results
    disp('Average Confusion Matrix:');
    disp(average_ConfMatrix_test);

    disp('Average Accuracy:');
    disp(mean(diag(average_ConfMatrix_test)));
    
function Mp3TranscodingDetection(AlphaY,SVs,Bias,Parameters,nSV,nLabel)

    %Initialize parameters
    N_features = 100; %number of features to consider
    N_items = 2512;   %total number of songs - use all of them for testing Transcoding
    N_iterations = 5; %number of iterations for SVM, each with different divisions of test & train set

    %Load feature data
    load('mat\data_128-320');
    load('mat\data_192-320');
    load('mat\data_320');

    %Keep only the first N_features, and then normalize each instance to the range -1 to 1.
    n_f128_320 = Scale(f128_320(1:N_features,:), -1, 1);
    n_f192_320 = Scale(f192_320(1:N_features,:), -1, 1);
    n_f320 = Scale(f320(1:N_features,:), -1, 1);

    %Initialize average confusion matrix across the number of SVM iterations
    average_ConfMatrix_test=zeros(5,5);

    %Loop to run SVM N_iterations times
    for iteration=1:N_iterations

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Construct test samples, as well as test labels
        test_sample = cat(2, n_f128_320,        n_f192_320,        n_f320);
        test_label  = cat(2, 1*ones(1,N_items), 2*ones(1,N_items), 5*ones(1,N_items));                          

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Use the results of training to classify the Test sample & obtain performance
        [ClassRate_test,DecisionValue_test,Ns_test,ConfMatrix_test,PreLabels_test] = SVMTest(test_sample,test_label,AlphaY,SVs,Bias,Parameters,nSV,nLabel);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Record confusion matrix results of this iteration
        average_ConfMatrix_test = average_ConfMatrix_test + ConfMatrix_test / N_iterations;
    end

    %Display Results
    disp('Average Confusion Matrix:');
    disp(average_ConfMatrix_test);

    disp('Average Accuracy:');
    disp(mean(diag(average_ConfMatrix_test)));        
